import 'package:angel_framework/angel_framework.dart';
import 'package:zap_server/src/zap_function.dart';
import 'package:zap_architecture/zap_architecture.dart';

/// Created by Musa Usman on 18.01.2021
/// Copyright © 2021 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +1 307-459-1575

class Router {
  final Angel app;

  Router(this.app);

  get(String endpoint, ZAPFunction function) {
    app.get(endpoint, (req, res) async => res.complete(await function()));
  }

  post(String endpoint, ZAPFunction function) {
    app.post(endpoint, (req, res) async => res.complete(await function()));
  }

  put(String endpoint, ZAPFunction function) {
    app.put(endpoint, (req, res) async => res.complete(await function()));
  }

  delete(String endpoint, ZAPFunction function) {
    app.delete(endpoint, (req, res) async => res.complete(await function()));
  }
}

extension CompleteFunction on ResponseContext {
  void complete(Response response) {
    Map<String, dynamic> json = {
      //success
      "success": response.success,
      //data
      if (response.success) "data": response.data,
      //error_message
      if (response.success == false) "error_message": response.errorMessage,
      //friendly_error_message
      if (response.success == false)
        "friendly_error_message": response.friendlyErrorMessage,
      //stack_trace
      if (response.success == false)
        "stack_trace": StackTrace.current.toString().replaceAll("\n", "\r"),
    };

    serialize(json);
  }
}
