import 'dart:async';
import 'dart:io';
import 'package:angel_framework/http.dart';
import 'package:angel_framework/angel_framework.dart' hide Router;
import 'package:zap_architecture/zap_architecture.dart';
import 'package:zap_server/src/ip_address.dart';
import 'package:zap_server/src/router.dart';

/// Created by Musa Usman on 18.01.2021
/// Copyright © 2021 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +1 307-459-1575

abstract class ZAPServer {
  DateTime startTime;

  Angel app;
  AngelHttp http;
  Router router;

  int port;
  String serverAddress;

  Future<void> start({int portNumber, bool isReleaseMode = false}) async {
    try {
      startTime = DateTime.now();

      app = Angel();
      http = AngelHttp(app);
      router = Router(app);

      await prepare();
      await route(router);

      port = portNumber ?? 8888;

      serverAddress = isReleaseMode ? "127.0.0.1" : await IPAddress.get();

      await http.startServer(serverAddress, port);

      //404 Error Handler
      app.fallback((req, res) {
        Response response = routeNotFound(req.path);
        res.complete(response);
      });
    } catch (e, s) {
      print("ZAPServer: Init: UnexpectedError: $e\n$s");
      exit(1);
    }
  }

  FutureOr<void> prepare() {}

  FutureOr<void> route(Router router);

  Response routeNotFound(String endpoint);
}
