import 'dart:math';
import 'package:zap_architecture/zap_architecture.dart';

/// Created by Musa Usman on 14.09.2020
/// Copyright © 2020 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +92 324 9066001

void main() async {
  Response<String> response = await _fetchData();

  if (response.success) {
    print(
        "RESPONSE RECEIVED: ${response.data}"); // RESPONSE RECEIVED: Awesome Data
    print(
        "RESPONSE DATA TYPE: ${response.data.runtimeType}"); // RESPONSE DATA TYPE: String
  } else {
    print(
        "RESPONSE RECEIVED FAILURE: ${response.friendlyErrorMessage}"); // RESPONSE RECEIVED FAILURE: Try again, you have a 50% chance....
    print(
        "RESPONSE RECEIVED FAILURE: DEBUG INFO: ${response.errorMessage}"); // RESPONSE RECEIVED FAILURE: Try again, you have a 50% chance....
  }
}

//50% chance of receiving successful response.
Future<Response<String>> _fetchData() async {
  int rand = await Future.delayed(
    Duration(seconds: 2),
    () => Random().nextInt(50),
  );

  bool randomSuccessful = rand >= 50;

  if (randomSuccessful)
    //return Response.success(9); This will not work, your IDE will not let you send an int value where String is needed.
    return Response.success("Awesome Data");
  else
    return Response.error(
        friendlyErrorMessage:
            "Try again, you have a 50% chance of getting a successful response.");
}
